#!/usr/bin/env python
# coding: latin-1
# Autor:   Ingmar Stapel
# Datum:   20160731
# Version:   2.0
# Homepage:   http://custom-build-robots.com
# Programm fuer die Entfernungsmessung mit zwei
# SRF08 Ultraschall Sensoren hinter und vor dem
# Roboter-Auto.

import smbus
import time
import subprocess
import os

# Threads Bibliothek importieren
from threading import Thread

# Das Programm L298NHBridge.py wird als Modul geladen.
import L298NHBridge as HBridge

# Das Programm ledmatrix.py wird als Modul geladen. 
import ledmatrix as matrix

# Setzen des SMBus Geraetes.
dev = smbus.SMBus(1)

# stopp Bedingung für die While Schleifen der Threads.
stopp = 0

# setzt die initiale Geschwindigkeit auf 0.
speed_l = 0
speed_r = 0

# Die Buchstaben f, r und s legen die Richtung fest in die das 
# Roboter-Auto fahren soll.
# f = forward
# r = reverse
# s = stopp
direction = "s"
global dist
dist = 0
global time_stopp

# Diese Funktion steuert das Roboter-Auto bis kurz vor das Hindernis.
def move_robot():
   global stopp
   global speed_l
   global speed_r
   
   # So lange stopp = 0 arbeitet dieser Thread   in der
   # Endlosschleife.
   while stopp == 0:
      HBridge.setMotorLeft(speed_l)
      HBridge.setMotorRight(speed_r)
      time.sleep(0.01)            
      
# Initialisierung der Entfernungmessung mit den Eingabevariablen.
def start_ranging(sensor_id, mode, sensor_gain, sensor_range):
   try:
      #dev.write_byte_data(sensor_id, 1, sensor_gain)
      #dev.write_byte_data(sensor_id, 2, sensor_range)
      dev.write_byte_data(sensor_id, 0, mode)
      return -1
   except ValueError: 
      sys.exit()


# Diese Funktion gibt die gemessene Entfernung zurueck.
def distance(sensor_id):
   highByte2 = dev.read_byte_data(sensor_id, 2)
   lowByte3 = dev.read_byte_data(sensor_id, 3)
   calc_range = (highByte2 << 8) + lowByte3
   return calc_range

def control():
   global stopp
   global speed_l
   global speed_r
   
   # setzen der initial Werte für den Ultraschall
   # Sensor vorne am Auto.
   sensor_v = 0x70
   # setzen der initial Werte für den Ultraschall
   # Sensor hinten am Auto.
   sensor_h = 0x72
   # Modus fuer die Messung festlegen.
   mode = 0x51
   intervall = 0.07
   sensor_id = sensor_v
   speed = 0
   # f steht fuer die Fahrtrichtung forward (vorwaerts).
   # r steht für die Fahrtichtung reverse (zurueck).
   # Vorbelegt wird die Fahrtrichtung mit f.
   direction = "f"

   try:
      min_dist=input("Eingabe minimale Entfernung in cm (z. B. 10):")
      sensor_gain=input("Eingabe Sensor GAIN (z. B. 6):")
      sensor_range=input("Eingabe Sensor RANGE (z. B. 430 mm):")
      laufzeit=input("Laufzeit des Programmes in Minuten: (z. B. 0.5):")   
   except ValueError: 
      sys.exit()

   # Berechnen der vom Anwender eingebenen Laufzeit in Sekunden.
   time_stopp = laufzeit*60 + time.time()

   # Zeigt den Pfeil fuer das vorwaerts Fahren an.
   matrix.display_pixels("arrow")
   matrix.display_rotate(90)   
      
   # Dieser Thread arbeitet in einer Endlos-Schleife 
   # solange bis stopp auf 1 gesetzt wird.
   while stopp == 0:
      # Wenn die vom Anwender eingebene Laufzeit des
      # Programmes abgelaufen ist wird das Programm
      # automatisch beendet.
      if time_stopp < time.time():
         stopp = 1
         # Zeigt das Programm Ende Symbol an auf der LED-Matrix.
         matrix.display_pixels("finish")
   
      # Das Programm startet die Entfernungsmessung ueber
      # die Funktion start_ranging.
      start_ranging(sensor_id, mode, sensor_gain, sensor_range)

      # Wartezeit damit der SRF08 Ultraschall Sensor die 
      # Messung durchfuehren kann
      time.sleep(intervall)      

      # Aufruf der Funktion distance um die letzte gemessene
      # Entfernung zu erhalten.
      act_dist = distance(sensor_id)
      
      # Minimale Plausibilitaetspruefung ob die gemessene
      # Entfernung realistisch ist.
      if act_dist != 0 and act_dist <= 2000:
         dist = act_dist

      # Steuerung der Geschwindigkeit bis zum Hindernis in 
      # drei Stufen. Die letzte Stufe direkt vor dem Hinternis
      # stoppt das Roboter-Auto und laest es rueckwaerts
      # weiter fahren.
      if min_dist + 40 <= dist <= min_dist + 100:
         speed = 0.5
      elif min_dist + 25 <= dist <= min_dist + 40:
         speed = 0.4
      elif min_dist <= dist <= min_dist + 25:
         speed = 0.35
      elif 0 <= dist <= min_dist:
         speed = 0
         if direction == "f":
            sensor_id = sensor_h   
            direction = "r"
            # Zeigt den Pfeil fuer das rueckwaerts Fahren an.
            matrix.display_pixels("arrow")
            matrix.display_rotate(270)   
         elif direction == "r":
            sensor_id = sensor_v   
            direction = "f"
            # Zeigt den Pfeil fuer das vorwaerts Fahren an.
            matrix.display_pixels("arrow")
            matrix.display_rotate(90)               
      else:
         # Damit das Auto bei einer Entfernung groesser
         # 100 + min_dist losfaehrt wird diese Abfrage
         # gebraucht die speed von 0 auf 0.5 setzt.
         speed = 0.5
            
      # setzen der Fahrtrichtung fuer vorwaerts und 
      # rueckwaerts.
      if direction == "f":
         speed_l = speed
         speed_r = speed
      else:
         speed_l = speed * -1
         speed_r = speed * -1

      # Ausgabe der aktuellen Informationen.
      os.system('clear')         
      print "Abstand:          ", dist, " cm"
      print "SensorID:         ", sensor_id
      print "Fahrtrichtung:    ", direction
      print "Geschwindigkeit:  ", speed_l
      print "Programmlaufzeit: ",round(time_stopp - time.time(),1)," s"     
      time.sleep(0.04)
         
# Starten des Threads der den Motortreiber steuert.
t_move_robot = Thread(target=move_robot)
t_move_robot.start()

# Starten des Hauptthreads der die Entfernungsmessung und 
# Steuerung der Richtung und Geschwindigkeit der Motoren
# bestimmt.
t_control = Thread(target=control)
t_control.start()

# Beide Threads zusammen fuehren damit diese beim Beenden
# gleichzeitig sich beenden.
t_move_robot.join()
t_control.join()
# Ende des Programmes
